@@ -9,19 +9,25 @@ import shutil
 import subprocess
 import tempfile
 import warnings
+from collections import OrderedDict
+from dataclasses import dataclass, field
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Callable, Optional, Union
+from typing import TYPE_CHECKING, Any, Callable, Optional, Union, no_type_check
 
-import packaging
 import subprocess_tee
 from packaging.version import Version
 
 from ansible_compat.config import (
     AnsibleConfig,
     ansible_collections_path,
+    ansible_version,
     parse_ansible_version,
 )
-from ansible_compat.constants import MSG_INVALID_FQRL
+from ansible_compat.constants import (
+    MSG_INVALID_FQRL,
+    RC_ANSIBLE_OPTIONS_ERROR,
+    REQUIREMENT_LOCATIONS,
+)
 from ansible_compat.errors import (
     AnsibleCommandError,
     AnsibleCompatError,
@@ -47,16 +53,104 @@ class AnsibleWarning(Warning):
     """Warnings related to Ansible runtime."""
 
 
+@dataclass
+class Collection:
+    """Container for Ansible collection information."""
+
+    name: str
+    version: str
+    path: Path
+
+
+class CollectionVersion(Version):
+    """Collection version."""
+
+    def __init__(self, version: str) -> None:
+        """Initialize collection version."""
+        # As packaging Version class does not support wildcard, we convert it
+        # to "0", as this being the smallest version possible.
+        if version == "*":
+            version = "0"
+        super().__init__(version)
+
+
+@dataclass
+class Plugins:  # pylint: disable=too-many-instance-attributes
+    """Dataclass to access installed Ansible plugins, uses ansible-doc to retrieve them."""
+
+    runtime: "Runtime"
+    become: dict[str, str] = field(init=False)
+    cache: dict[str, str] = field(init=False)
+    callback: dict[str, str] = field(init=False)
+    cliconf: dict[str, str] = field(init=False)
+    connection: dict[str, str] = field(init=False)
+    httpapi: dict[str, str] = field(init=False)
+    inventory: dict[str, str] = field(init=False)
+    lookup: dict[str, str] = field(init=False)
+    netconf: dict[str, str] = field(init=False)
+    shell: dict[str, str] = field(init=False)
+    vars: dict[str, str] = field(init=False)  # noqa: A003
+    module: dict[str, str] = field(init=False)
+    strategy: dict[str, str] = field(init=False)
+    test: dict[str, str] = field(init=False)
+    filter: dict[str, str] = field(init=False)  # noqa: A003
+    role: dict[str, str] = field(init=False)
+    keyword: dict[str, str] = field(init=False)
+
+    @no_type_check
+    def __getattribute__(self, attr: str):  # noqa: ANN204
+        """Get attribute."""
+        if attr in {
+            "become",
+            "cache",
+            "callback",
+            "cliconf",
+            "connection",
+            "httpapi",
+            "inventory",
+            "lookup",
+            "netconf",
+            "shell",
+            "vars",
+            "module",
+            "strategy",
+            "test",
+            "filter",
+            "role",
+            "keyword",
+        }:
+            try:
+                result = super().__getattribute__(attr)
+            except AttributeError as exc:
+                if ansible_version() < Version("2.14") and attr in {"filter", "test"}:
+                    msg = "Ansible version below 2.14 does not support retrieving filter and test plugins."
+                    raise RuntimeError(msg) from exc
+                proc = self.runtime.run(
+                    ["ansible-doc", "--json", "-l", "-t", attr],
+                )
+                data = json.loads(proc.stdout)
+                if not isinstance(data, dict):  # pragma: no cover
+                    msg = "Unexpected output from ansible-doc"
+                    raise AnsibleCompatError(msg) from exc
+                result = data
+        else:
+            result = super().__getattribute__(attr)
+
+        return result
+
+
+# pylint: disable=too-many-instance-attributes
 class Runtime:
     """Ansible Runtime manager."""
 
-    _version: Optional[packaging.version.Version] = None
+    _version: Optional[Version] = None
+    collections: OrderedDict[str, Collection] = OrderedDict()
     cache_dir: Optional[Path] = None
     # Used to track if we have already initialized the Ansible runtime as attempts
     # to do it multiple tilmes will cause runtime warnings from within ansible-core
     initialized: bool = False
+    plugins: Plugins
 
-    # pylint: disable=too-many-arguments
     def __init__(
         self,
         project_dir: Optional[Path] = None,
@@ -92,6 +186,7 @@ class Runtime:
         self.isolated = isolated
         self.max_retries = max_retries
         self.environ = environ or os.environ.copy()
+        self.plugins = Plugins(runtime=self)
         # Reduce noise from paramiko, unless user already defined PYTHONWARNINGS
         # paramiko/transport.py:236: CryptographyDeprecationWarning: Blowfish has been deprecated
         # https://github.com/paramiko/paramiko/issues/2038
@@ -122,11 +217,50 @@ class Runtime:
             formatted: bool = False,  # noqa: ARG001
         ) -> None:
             """Override ansible.utils.display.Display.warning to avoid printing warnings."""
-            warnings.warn(msg, category=AnsibleWarning, stacklevel=2)
+            warnings.warn(
+                message=msg,
+                category=AnsibleWarning,
+                stacklevel=2,
+                source={"msg": msg},
+            )
 
         # Monkey patch ansible warning in order to use warnings module.
         Display.warning = warning
 
+    def load_collections(self) -> None:
+        """Load collection data."""
+        self.collections = OrderedDict()
+        no_collections_msg = "None of the provided paths were usable"
+
+        proc = self.run(["ansible-galaxy", "collection", "list", "--format=json"])
+        if proc.returncode == RC_ANSIBLE_OPTIONS_ERROR and (
+            no_collections_msg in proc.stdout or no_collections_msg in proc.stderr
+        ):
+            _logger.debug("Ansible reported no installed collections at all.")
+            return
+        if proc.returncode != 0:
+            _logger.error(proc)
+            msg = f"Unable to list collections: {proc}"
+            raise RuntimeError(msg)
+        data = json.loads(proc.stdout)
+        if not isinstance(data, dict):
+            msg = f"Unexpected collection data, {data}"
+            raise TypeError(msg)
+        for path in data:
+            for collection, collection_info in data[path].items():
+                if not isinstance(collection, str):
+                    msg = f"Unexpected collection data, {collection}"
+                    raise TypeError(msg)
+                if not isinstance(collection_info, dict):
+                    msg = f"Unexpected collection data, {collection_info}"
+                    raise TypeError(msg)
+
+                self.collections[collection] = Collection(
+                    name=collection,
+                    version=collection_info["version"],
+                    path=path,
+                )
+
     def _ensure_module_available(self) -> None:
         """Assure that Ansible Python module is installed and matching CLI version."""
         ansible_release_module = None
@@ -137,7 +271,7 @@ class Runtime:
             msg = "Unable to find Ansible python module."
             raise RuntimeError(msg)
 
-        ansible_module_version = packaging.version.parse(
+        ansible_module_version = Version(
             ansible_release_module.__version__,
         )
         if ansible_module_version != self.version:
@@ -175,7 +309,7 @@ class Runtime:
         if self.cache_dir:
             shutil.rmtree(self.cache_dir, ignore_errors=True)
 
-    def run(
+    def run(  # ruff: disable=PLR0913
         self,
         args: Union[str, list[str]],
         *,
@@ -193,6 +327,13 @@ class Runtime:
             run_func: Callable[..., CompletedProcess] = subprocess_tee.run
         else:
             run_func = subprocess.run
+        env = self.environ if env is None else env.copy()
+        # Presence of ansible debug variable or config option will prevent us
+        # from parsing its JSON output due to extra debug messages on stdout.
+        env["ANSIBLE_DEBUG"] = "0"
+
+        # https://github.com/ansible/ansible-lint/issues/3522
+        env["ANSIBLE_VERBOSE_TO_STDERR"] = "True"
 
         for _ in range(self.max_retries + 1 if retry else 1):
             result = run_func(
@@ -201,21 +342,22 @@ class Runtime:
                 check=False,
                 stdout=subprocess.PIPE,
                 stderr=subprocess.PIPE,
-                env=env or self.environ,
+                env=env,
                 cwd=str(cwd) if cwd else None,
             )
             if result.returncode == 0:
                 break
-            _logger.debug("Environment: %s", env or self.environ)
-            _logger.warning(
-                "Retrying execution failure %s of: %s",
-                result.returncode,
-                " ".join(args),
-            )
+            _logger.debug("Environment: %s", env)
+            if retry:
+                _logger.warning(
+                    "Retrying execution failure %s of: %s",
+                    result.returncode,
+                    " ".join(args),
+                )
         return result
 
     @property
-    def version(self) -> packaging.version.Version:
+    def version(self) -> Version:
         """Return current Version object for Ansible.
 
         If version is not mentioned, it returns current version as detected.
@@ -242,9 +384,9 @@ class Runtime:
 
         The lower limit is inclusive and the upper one exclusive.
         """
-        if lower and self.version < packaging.version.Version(lower):
+        if lower and self.version < Version(lower):
             return False
-        if upper and self.version >= packaging.version.Version(upper):
+        if upper and self.version >= Version(upper):
             return False
         return True
 
@@ -272,7 +414,7 @@ class Runtime:
         # if the range requires a pre-release, we need to manuall add the --pre
         # flag when needed.
         matches = version_re.search(str(collection))
-        if matches and Version(matches[1]).is_prerelease:
+        if matches and CollectionVersion(matches[1]).is_prerelease:
             cmd.append("--pre")
 
         cpaths: list[str] = self.config.collections_paths
@@ -326,7 +468,7 @@ class Runtime:
                 )
 
     # pylint: disable=too-many-branches
-    def install_requirements(  # noqa: C901,PLR0912
+    def install_requirements(  # noqa: C901
         self,
         requirement: Path,
         *,
@@ -346,6 +488,12 @@ class Runtime:
             msg = f"{requirement} file is not a valid Ansible requirements file."
             raise InvalidPrerequisiteError(msg)
 
+        if isinstance(reqs_yaml, dict):
+            for key in reqs_yaml:
+                if key not in ("roles", "collections"):
+                    msg = f"{requirement} file is not a valid Ansible requirements file. Only 'roles' and 'collections' keys are allowed at root level. Recognized valid locations are: {', '.join(REQUIREMENT_LOCATIONS)}"
+                    raise InvalidPrerequisiteError(msg)
+
         if isinstance(reqs_yaml, list) or "roles" in reqs_yaml:
             cmd = [
                 "ansible-galaxy",
@@ -420,13 +568,24 @@ class Runtime:
         # are part of Tower specification
         # https://docs.ansible.com/ansible-tower/latest/html/userguide/projects.html#ansible-galaxy-support
         # https://docs.ansible.com/ansible-tower/latest/html/userguide/projects.html#collections-support
-        for req_file in [
-            "requirements.yml",
-            "roles/requirements.yml",
-            "collections/requirements.yml",
-        ]:
+        for req_file in REQUIREMENT_LOCATIONS:
             self.install_requirements(Path(req_file), retry=retry, offline=offline)
 
+        galaxy_path = Path("galaxy.yml")
+        if galaxy_path.exists():
+            data = yaml_from_file(galaxy_path)
+            if isinstance(data, dict) and "dependencies" in data:
+                for name, required_version in data["dependencies"].items():
+                    _logger.info(
+                        "Provisioning collection %s:%s from galaxy.yml",
+                        name,
+                        required_version,
+                    )
+                    self.install_collection(
+                        f"{name}:{required_version}",
+                        destination=destination,
+                    )
+
         if self.cache_dir:
             destination = self.cache_dir / "collections"
         for name, min_version in required_collections.items():
@@ -474,6 +633,8 @@ class Runtime:
                 role_name_check=role_name_check,
                 ignore_errors=True,
             )
+        # reload collections
+        self.load_collections()
 
     def require_collection(
         self,
@@ -519,10 +680,10 @@ class Runtime:
 
                 with mpath.open(encoding="utf-8") as f:
                     manifest = json.loads(f.read())
-                    found_version = packaging.version.parse(
+                    found_version = CollectionVersion(
                         manifest["collection_info"]["version"],
                     )
-                    if version and found_version < packaging.version.parse(version):
+                    if version and found_version < CollectionVersion(version):
                         if install:
                             self.install_collection(f"{name}:>={version}")
                             self.require_collection(name, version, install=False)
